/*******************************************************************************
 *	BDir.cc	BEAM BDir access class
 *			T.Barnaby,	BEAM Ltd,	8/10/96
 *******************************************************************************
 */
#include	<BDir.h>
#include	<dirent.h>
#include	<stdlib.h>
#include	<errno.h>
#include	<string.h>

static BString wildString;

static int wild(const dirent* e){
	BString	s = e->d_name;

	return s.compareWild(wildString);
}

BDir::BDir(){
	osort = 0;
}

BDir::BDir(BString name){
	osort = 0;
	odirname = name;
	oerror = read();
}

BDir::~BDir(){
	clear();
}

void BDir::clear(){
	BIter	i;
	
	for(i = begin(); !isEnd(i); del(i)){
		delete get(i);
	}
}

BError BDir::open(BString name){
	odirname = name;
	oerror = read();
	return oerror;
}

BError BDir::error(){
	return oerror;
}

void BDir::setWild(BString wild){
	owild = wild;
	read();
}

void BDir::setSort(int sort){
	osort = sort;
	read();
}

BError BDir::read(){
	BError		err;
	struct dirent**	e;
	int		n;
	int		v;
	int		(*sel)(const struct dirent*) = 0;
	int		(*cmp)(const void*, const void*) = 0;

	clear();

	if(owild != ""){
		wildString = owild;
		sel = wild;
	}
	if(osort == 1)
		cmp = alphasort;

	if((n = scandir(odirname, &e, sel, cmp)) < 0)
		return BError(strerror(errno));

	for(v = 0; v < n; v++){
		append(e[v]);
	}
	free(e);
	return err;
}

BString BDir::entryName(BIter i){
	return get(i)->d_name;
}

struct stat BDir::entryStat(BIter i){
	struct stat	s = {0};

	lstat(odirname + "/" + entryName(i), &s);
	return s;
}

struct stat64 BDir::entryStat64(BIter i){
	struct stat64	s = {0};

	lstat64(odirname + "/" + entryName(i), &s);
	return s;
}
